#!/usr/bin/env python
# -*- coding: utf-8 -*-

#     Copyright 2010 Sergiy S. Kolesnikov
#
#     This file is part of csv2itdb.
#
#     csv2itdb is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     csv2itdb is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with csv2itdb.  If not, see <http://www.gnu.org/licenses/>.

# TODO:
# comments
# add the URL of the project to the comments
# --out: output file
# --encoding: CSV encoding
# --help

'''
CSV to iTestDB converter version %(version)s

Usage:
python csv2itdb.py <file.csv>

Format of the CSV file:
Question_Name,Question,Explanation,Answer_1,Answer_2,...

If Question_Name is an empty string, Qustion will be used instead.

Encoding of the CSV file must be UTF-8 for now.

csv2itdb accepts CSV dialect defined in the csv.excel Python class:
  delimiter = ','
  quotechar = '"'
  doublequote = True
  skipinitialspace = False
  lineterminator = '\\r\\n'
  quoting = QUOTE_MINIMAL
See http://docs.python.org/library/csv.html#dialects-and-formatting-parameters
for more information.

Generated iTest DB version is %(dbVersion)s for iTest version %(itestVersion)s
'''

import os
import sys
import getopt
import codecs
import csv
import time

version = 0.1

# TODO make it a dictionary and update with values from optget
itestVersionOpt = 1.42
dbVersionOpt = 1.4
inputEncodingOpt = 'utf-8'
outputEncodingOpt = 'utf-8'
correctAnswerNumOpt = 1

itdbHeader = u'''[ITEST_VERSION]
%(itestVersion)s
[ITEST_DB_VERSION]
%(dbVersion)s
[DB_NAME]
%(dbName)s
[DB_DATE]
%(date)s
[DB_DATE_ULSD]
true
[DB_COMMENTS]
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd"><html><head><meta name="qrichtext" content="1" /><style type="text/css">p, li { white-space: pre-wrap; }</style></head><body style=" font-family:'Sans Serif'; font-size:11pt; font-weight:400; font-style:normal;"><p style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;">%(comments)s</p></body></html>
[DB_QNUM]
%(questionsNumber)i
[DB_SNUM]
0
[DB_CNUM]
0
[DB_FLAGS]
--------------------
[DB_F0]

[DB_F1]

[DB_F2]

[DB_F3]

[DB_F4]

[DB_F5]

[DB_F6]

[DB_F7]

[DB_F8]

[DB_F9]

[DB_F10]

[DB_F11]

[DB_F12]

[DB_F13]

[DB_F14]

[DB_F15]

[DB_F16]

[DB_F17]

[DB_F18]

[DB_F19]

[DB_FLAGS_END]'''

class Answers(object):
    def __init__(self, ans, rNum=correctAnswerNumOpt):
        self.answers = ans
        self.correctAnswerNum = rNum

    def getAnswersNum(self):
        return len(self.answers)

    def __str__(self):
        return unicode(self).encode(outputEncodingOpt)
    
    def __unicode__(self):
        answersString = u''
        for a in self.answers:
            answersString += a + '\n'
        return answersString[0:-1] # trim the last newline

class Question(object):
    def __init__(self, name=u'', question=u'', explanation=u'', answers=Answers([u''])):
        if name == u'':
            self.name = question
        else:
            self.name = name
        self.question = question
        self.explanation = explanation
        self.answers = answers

    def __str__(self):
        return unicode(self).encode(outputEncodingOpt)

    def __unicode__(self):
        return u'''[Q_NAME]
%(name)s
[Q_FLAG]
-1
[Q_GRP]

[Q_DIF]
0
[Q_TEXT]
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd"><html><head><meta name="qrichtext" content="1" /><style type="text/css">p, li { white-space: pre-wrap; }</style></head><body style=" font-family:'Sans Serif'; font-size:11pt; font-weight:400; font-style:normal;"><p style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;">%(question)s</p></body></html>
[Q_ANS]
0
%(correctAnswerNum)i
%(answersNum)i
%(answers)s
[Q_EXPL]
%(explanation)s
[Q_ICCNT]
0
0
[Q_HID]
false
[Q_SVG]
0''' % {'name':self.name, \
            'question':self.question, \
            'correctAnswerNum':self.answers.correctAnswerNum, \
            'answersNum':self.answers.getAnswersNum(), \
            'answers':unicode(self.answers), \
            'explanation':self.explanation}

class UTF8Recoder(object):
    """
    Iterator that reads an encoded stream and reencodes the input to UTF-8.
    """

    def __init__(self, f, encoding):
        self.reader = codecs.getreader(encoding)(f)

    def __iter__(self):
        return self

    def next(self):
        return self.reader.next().encode("utf-8")

class UnicodeReader(object):
    """
    A CSV reader which will iterate over lines in the CSV file "f",
    which is encoded in the given encoding.
    """

    def __init__(self, f, encoding="utf-8", dialect=csv.excel, **kwds):
        f = UTF8Recoder(f, encoding)
        self.reader = csv.reader(f, dialect=dialect, **kwds)

    def next(self):
        row = self.reader.next()
        return [unicode(s, "utf-8") for s in row]

    def __iter__(self):
        return self

def lineNumbers(filePath):
    file = open(filePath, 'rU')
    ln = 0
    for line in file:
        ln += 1
    file.close()
    return ln

def main():
    try:
        options, operands = getopt.getopt(sys.argv[1:], '', ())
        if len(operands) != 1: 
            raise getopt.GetoptError("One CSV file must be specified")
        csvFilePath = operands[0]
    except getopt.GetoptError, e:
        print('Error: ' + e.args[0])
        print(__doc__ % {'version':version,
                         'dbVersion':dbVersionOpt, 
                         'itestVersion':itestVersionOpt})
        return
    csvFile = open(csvFilePath, 'rb')
    csvReader = UnicodeReader(csvFile, inputEncodingOpt)
    print(itdbHeader % {'itestVersion':itestVersionOpt,
                        'dbVersion':dbVersionOpt,
                        'dbName':os.path.basename(csvFilePath),
                        'date':time.strftime('%Y.%m.%d-%H:%M',time.localtime()),
                        'comments':"Generated with csv2itdb.py",
                        'questionsNumber':lineNumbers(csvFilePath)})
    for row in csvReader:
        name = row[0]
        question = row[1]
        explanation = row[2]
        answers = Answers(row[3:])
        print(unicode(Question(name, question, explanation, answers))
              .encode(outputEncodingOpt))

        
            
if __name__ == '__main__':
    main()
